<?php # $Id: serendipity_event_spamblock.php,v 1.6 2004/09/18 19:46:52 tomsommer Exp $

switch ($serendipity['lang']) {
    default:
        @define('PLUGIN_EVENT_SPAMBLOCK_TITLE', 'Spam Protector');
        @define('PLUGIN_EVENT_SPAMBLOCK_DESC', 'A varity of methods to prevent comment spam');
        @define('PLUGIN_EVENT_SPAMBLOCK_ERROR_BODY', 'Spam Prevention: Invalid message.');
        @define('PLUGIN_EVENT_SPAMBLOCK_ERROR_IP', 'Spam Prevention: You cannot post a comment so soon after submitting another one.');
        @define('PLUGIN_EVENT_SPAMBLOCK_ERROR_RBL', 'Spam Prevention: The IP of the computer you are posting from, is listed as an open relay.');
        @define('PLUGIN_EVENT_SPAMBLOCK_ERROR_KILLSWITCH', 'This blog is in "Emergency Comment Blockage Mode", please come back another time');
        @define('PLUGIN_EVENT_SPAMBLOCK_BODYCLONE', 'Do not allow duplicate comments');
        @define('PLUGIN_EVENT_SPAMBLOCK_BODYCLONE_DESC', 'Do not allow users to submit a comment which contains the same body as an already submitted comment');
        @define('PLUGIN_EVENT_SPAMBLOCK_KILLSWITCH', 'Emergency comment shutdown');
        @define('PLUGIN_EVENT_SPAMBLOCK_KILLSWITCH_DESC', 'Temporarily disable comments for all entries. Useful if your blog is under spam attack.');
        @define('PLUGIN_EVENT_SPAMBLOCK_IPFLOOD', 'IP block interval');
        @define('PLUGIN_EVENT_SPAMBLOCK_IPFLOOD_DESC', 'Only allow an IP to submit a comment every n minutes. Useful to prevent comment floods.');
        @define('PLUGIN_EVENT_SPAMBLOCK_RBL', 'Block comments from RBL-listed hosts');
        @define('PLUGIN_EVENT_SPAMBLOCK_RBL_DESC', 'Blocks comments based on provided RPL lists. Avoid lists with dynamic hosts.');
        break;
}

class serendipity_event_spamblock extends serendipity_event
{
var $services;

    function introspect(&$propbag)
    {
        global $serendipity;

        $propbag->add('name',          PLUGIN_EVENT_SPAMBLOCK_TITLE);
        $propbag->add('description',   PLUGIN_EVENT_SPAMBLOCK_DESC);
        $propbag->add('event_hooks',    array(
            'frontend_saveComment' => true
        ));
        $propbag->add('configuration', array('bodyclone', 'killswitch', 'ipflood', 'rbl'));
    }

    function introspect_config_item($name, &$propbag)
    {
        switch($name) {
            case 'bodyclone':
                $propbag->add('type', 'boolean');
                $propbag->add('name', PLUGIN_EVENT_SPAMBLOCK_BODYCLONE);
                $propbag->add('description', PLUGIN_EVENT_SPAMBLOCK_BODYCLONE_DESC);
                $propbag->add('default', true);
                break;

            case 'killswitch':
                $propbag->add('type', 'boolean');
                $propbag->add('name', PLUGIN_EVENT_SPAMBLOCK_KILLSWITCH);
                $propbag->add('description', PLUGIN_EVENT_SPAMBLOCK_KILLSWITCH_DESC);
                $propbag->add('default', false);
                break;

            case 'ipflood':
                $propbag->add('type', 'string');
                $propbag->add('name', PLUGIN_EVENT_SPAMBLOCK_IPFLOOD);
                $propbag->add('description', PLUGIN_EVENT_SPAMBLOCK_IPFLOOD_DESC);
                $propbag->add('default', 2);
                break;

            case 'rbl':
                $propbag->add('type', 'string');
                $propbag->add('name', PLUGIN_EVENT_SPAMBLOCK_RBL);
                $propbag->add('description', PLUGIN_EVENT_SPAMBLOCK_RBL_DESC);
                $propbag->add('default', 'sbl-xbl.spamhaus.org, bl.spamcop.net');
                break;

            default:
                    return false;
        }

        return true;
    }


    function generate_content(&$title) {
        $title = PLUGIN_EVENT_SPAMBLOCK_TITLE;
    }

    function event_hook($event, &$bag, &$eventData, $addData = null) {
        global $serendipity;

        $hooks = &$bag->get('event_hooks');
        if (isset($hooks[$event])) {
            switch($event) {
                case 'frontend_saveComment':
                    if (!is_array($eventData) || serendipity_db_bool($eventData['allow_comments'])) {

                        if ( $this->get_config('killswitch', false) === true ) {
                            $eventData = array('allow_comments' => false);
                            $serendipity['messagestack']['comments'][] = PLUGIN_EVENT_SPAMBLOCK_ERROR_KILLSWITCH;
                            return false;
                        }

                        // Check for identical comments.
                        if ( $this->get_config('bodyclone', true) === true ) {
                            $query = "SELECT count(id) AS counter FROM {$serendipity['dbPrefix']}comments WHERE body = '" . serendipity_db_escape_string($addData['comment']) . "'";
                            $row   = serendipity_db_query($query, true);
                            if (is_array($row) && $row['counter'] > 0) {
                                $eventData = array('allow_comments' => false);
                                $serendipity['messagestack']['comments'][] = PLUGIN_EVENT_SPAMBLOCK_ERROR_BODY;
                                return false;
                            }
                        }

                        // Check last IP
                        if ( $this->get_config('ipflood', 2) != 0 ) {
                            $query = "SELECT max(timestamp) AS last_post FROM {$serendipity['dbPrefix']}comments WHERE ip = '" . serendipity_db_escape_string($_SERVER['REMOTE_ADDR']) . "'";
                            $row   = serendipity_db_query($query, true);
                            if (is_array($row) && $row['last_post'] > (time() - $this->get_config('ipflood', 2)*60)) {
                                $eventData = array('allow_comments' => false);
                                $serendipity['messagestack']['comments'][] = PLUGIN_EVENT_SPAMBLOCK_ERROR_IP;
                                return false;
                            }
                        }

                        // Check for IP listed in RBL
                        // Partly based on work done by James Seward (james@jamesoff.net)
                        if ( $this->get_config('rbl') != '' ) {
                            $remoteIP = $_SERVER['REMOTE_ADDR'];
                        #    $remoteIP = '81.70.69.193';

                            $rbls = explode(',', $this->get_config('rbl'));
                            if (preg_match("/([0-9]+)\.([0-9]+)\.([0-9]+)\.([0-9]+)/", $remoteIP, $res)) {
                                array_shift($res);
                                $reverseIP = implode('.', array_reverse($res));
                                foreach ($rbls as $rbl) {
                                    $rblhost = $reverseIP .'.'. trim($rbl);
                                    if (gethostbyname($rblhost) != $rblhost) {
                                        $eventData = array('allow_comments' => false);
                                        $serendipity['messagestack']['comments'][] = PLUGIN_EVENT_SPAMBLOCK_ERROR_RBL;
                                        return false;
                                    }
                                }               
                            }
                        }
                    }
                    return true;
                break;

                default:
                    return false;
                    break;
            }
        } else {
            return false;
        }
    }
}

/* vim: set sts=4 ts=4 expandtab : */
?>
