<?php # $Id: serendipity_event_xhtmlcleanup.php,v 1.6 2004/08/09 08:49:08 garvinhicking Exp $

switch ($serendipity['lang']) {
    case 'de':
        @define('PLUGIN_EVENT_XHTMLCLEANUP_NAME', 'bliche XHTML-Fehler beseitigen');
        @define('PLUGIN_EVENT_XHTMLCLEANUP_DESC', 'Korrigiert bliche Fehler, die beim XHTML-Markup der Eintrge gemacht werden knnen');
        break;

    case 'en':
    default:
        @define('PLUGIN_EVENT_XHTMLCLEANUP_NAME', 'Fix common XHTML errors');
        @define('PLUGIN_EVENT_XHTMLCLEANUP_DESC', 'Corrects common issues with XHTML markup in entries');
        break;
}

if (!function_exists('html_entity_decode')) {
    function html_entity_decode($given_html, $quote_style = ENT_QUOTES) {
        $trans_table = get_html_translation_table(HTML_SPECIALCHARS, $quote_style);
        if ($trans_table["'"] != '&#039;') { # some versions of PHP match single quotes to &#39;
          $trans_table["'"] = '&#039;';
        }

        return (strtr($given_html, array_flip($trans_table)));
    }
}

class serendipity_event_xhtmlcleanup extends serendipity_event
{
    var $cleanup_tag, $cleanup_checkfor, $cleanup_val;

    function introspect(&$propbag)
    {
        global $serendipity;

        $propbag->add('name',          PLUGIN_EVENT_XHTMLCLEANUP_NAME);
        $propbag->add('description',   PLUGIN_EVENT_XHTMLCLEANUP_DESC);
        $propbag->add('event_hooks',   array('frontend_display' => true));

        $this->markup_elements = array(
            array(
              'name'     => ENTRY_BODY,
              'element'  => 'body',
            ),
            array(
              'name'     => EXTENDED_BODY,
              'element'  => 'extended',
            ),
            array(
              'name'     => COMMENT,
              'element'  => 'comment',
            ),
            array(
              'name'     => HTML_NUGGET,
              'element'  => 'html_nugget',
            )
        );

        $conf_array = array();
        foreach($this->markup_elements as $element) {
            $conf_array[] = $element['name'];
        }
        $propbag->add('configuration', $conf_array);
    }


    function generate_content(&$title) {
        $title = PLUGIN_EVENT_XHTMLCLEANUP_NAME;
    }

    function introspect_config_item($name, &$propbag)
    {
        $propbag->add('type',        'boolean');
        $propbag->add('name',        $name);
        $propbag->add('description', sprintf(APPLY_MARKUP_TO, $name));
        $propbag->add('default',     'true');
        return true;
    }

    function event_hook($event, &$bag, &$eventData, $addData = null) {
        global $serendipity;

        $hooks = &$bag->get('event_hooks');

        if (isset($hooks[$event])) {
            switch($event) {
                case 'frontend_display':
                    foreach ($this->markup_elements as $temp) {
                        if (serendipity_db_bool($this->get_config($temp['name'], true)) && isset($eventData[$temp['element']])) {
                            $element = $temp['element'];
                            $this->cleanup_tag      = 'IMG';
                            $this->cleanup_checkfor = 'ALT';
                            $this->cleanup_val      = '';
                            // Basic cleanup (core s9y functionality)
                            $eventData[$element]    = xhtml_cleanup($eventData[$element]);
                            $eventData[$element]    = preg_replace_callback('@(<img.+/?>)@imsU', array($this, 'clean_tag'), $eventData[$element]);
                            $eventData[$element]    = preg_replace_callback("@<a(.*)href=(\"|')([^\"']+)(\"|')@isUm", array($this, 'clean_htmlspecialchars'), $eventData[$element]);
                        }
                    }

                    return true;
                    break;

                default:
                    return false;
            }

        } else {
            return false;
        }
    }

    // Takes an input tag and search for ommitted attributes. Expects a single tag (array, index 0)
    function clean_tag($data) {
        // Restore tags from preg_replace_callback buffer, as those can't be passed in the function header
        $tag      = &$this->cleanup_tag;
        $checkfor = &$this->cleanup_checkfor;
        $val      = &$this->cleanup_val;

        // Instead of nasty regex-mangling we use the XML parser to get the attribute list of our input tag
        $p = xml_parser_create();
        @xml_parse_into_struct($p, $data[0], $vals, $index);
        xml_parser_free($p);

        // Check if the xml parser returned anything useful
        if (is_array($vals) && isset($vals[0]) && $vals[0]['tag'] == $tag) {

            if (!empty($vals[0]['attributes'][$checkfor])) {
                // The attribute we search for already exists. Return original string.
                return $data[0];
            }

            // Assign the value we submitted for the attribute to insert
            $vals[0]['attributes'][$checkfor] = $val;

            // Reconstruct XHTML tag.
            $atts = ' ';
            foreach($vals[0]['attributes'] AS $att => $att_con) {
                $atts .= strtolower($att) . '="' . $att_con . '" ';
            }

            return '<' . strtolower($tag) . $atts . ' />';
        }

        return $data[0];
    }

    function clean_htmlspecialchars($given, $quote_style = ENT_QUOTES) {
        return '<a' . $given[1] . 'href=' . $given[2] . htmlspecialchars(html_entity_decode($given[3], $quote_style), $quote_style) . $given[4];
    }
}

/* vim: set sts=4 ts=4 expandtab : */
?>
