<?php
require_once(S9Y_INCLUDE_PATH . 'serendipity_functions_installer.inc.php');

// Setting this value to 'FALSE' is recommended only for SHARED BLOG INSTALLATIONS. This enforces all shared blogs with a common
// codebase to only allow upgrading, no bypassing and thus causing instabilities.
// This variable can also be set as $serendipity['UpgraderShowAbort'] inside serendipity_config_local.inc.php to prevent
// your setting being changed when updating serendipity in first place.
$showAbort  = (isset($serendipity['UpgraderShowAbort']) ? $serendipity['UpgraderShowAbort'] : true);

$abortLoc   = $serendipity['serendipityHTTPPath'] . 'serendipity_admin.php?serendipity[action]=ignore';
$upgradeLoc = $serendipity['serendipityHTTPPath'] . 'serendipity_admin.php?serendipity[action]=upgrade';

/* Functions which needs to be run if installed version is equal or lower */
$tasks = array('0.5.1' => 'serendipity_syncThumbs',
               '0.6.5' => 'serendipity_rebuildCategoryTree',
               '0.6.8' => 'serendipity_installFiles',
               '0.6.10' => 'serendipity_installFiles',
               '0.6.12' => 'serendipity_installFiles',
              );


/* Fetch SQL files which needs to be run */
$dir      = opendir(S9Y_INCLUDE_PATH . 'sql/');
$dbType   = $serendipity['dbType'] == 'mysqli' ? 'mysql' : $serendipity['dbType'];
$sqlfiles = array();
while (($file = readdir($dir)) !== false ) {
    if (preg_match('@db_update_(.*)_(.*)_'. $dbType .'.sql@', $file, $res)) {
        if (version_compare($res[1], serendipity_version($serendipity['versionInstalled'])) >= 0) {
            // Create a usable float as key index for our array to make it sortable
            $numbers = explode('.', $res[1]);
            $first   = $numbers[0];
            unset($numbers[0]);
            $sqlfiles[$first . '.' . @implode('', $numbers)] = $file;
        }
    }
}

@ksort($sqlfiles);

if ($serendipity['GET']['action'] == 'ignore') {
    /* Todo: Don't know what to put here? */

} elseif ($serendipity['GET']['action'] == 'upgrade') {

    /* Install SQL files */
    foreach ($sqlfiles as $sqlfile) {
        $sql = file_get_contents(S9Y_INCLUDE_PATH .'sql/'. $sqlfile);
        $sql = str_replace('{PREFIX}', $serendipity['dbPrefix'], $sql);
        preg_match_all("@(.*);@iUs", $sql, $res);
        foreach ($res[0] as $sql) {
                $r = serendipity_db_schema_import($sql);
                if (is_string($r)) {
                    $errors[] = $r;
                }
        }
    }


    /* Call functions */
    foreach ($tasks as $version => $function) {
        if (version_compare(serendipity_version($serendipity['versionInstalled']), $version, '<') ) {
            if (is_callable($function)) {
                echo 'Calling ' . $function . '...<br />';
                call_user_func($function);
            } else {
                $errors[] = 'Unable to call '. $function;
            }
        }
    }

    if (is_array($errors)) {
        echo DIAGNOSTIC_ERROR . '<br /><br />';
        echo '<span class="serendipity_msg_important">- ' . implode('<br />', $errors) . '</span><br /><br />';
    }

}

if (($showAbort && $serendipity['GET']['action'] == 'ignore') || $serendipity['GET']['action'] == 'upgrade') {
    $privateVariables = array();
    if (isset($serendipity['UpgraderShowAbort'])) {
        $privateVariables['UpgraderShowAbort'] = $serendipity['UpgraderShowAbort'];
    }

    $r = serendipity_updateLocalConfig(
           $serendipity['dbName'],
           $serendipity['dbPrefix'],
           $serendipity['dbHost'],
           $serendipity['dbUser'],
           $serendipity['dbPass'],
           $serendipity['dbType'],
           'serendipity_config_local.inc.php',
           null,
           $privateVariables
    );

    if ($serendipity['GET']['action'] == 'ignore') {
        echo SERENDIPITY_UPGRADER_YOU_HAVE_IGNORED;
    } elseif ($serendipity['GET']['action'] == 'upgrade') {
        printf(SERENDIPITY_UPGRADER_NOW_UPGRADED, $serendipity['version']);
    }
    echo '<br />';
    printf(SERENDIPITY_UPGRADER_RETURN_HERE, '<a href="'. $serendipity['serendipityHTTPPath'] .'">', '</a>');
    $_SESSION['serendipityAuthedUser'] = false;
    @session_destroy();
} else {
    echo SERENDIPITY_UPGRADER_WELCOME . '<br />';
    printf(SERENDIPITY_UPGRADER_PURPOSE . '<br />', $serendipity['versionInstalled']);
    printf(SERENDIPITY_UPGRADER_WHY, $serendipity['version']);

    if (sizeof($sqlfiles) > 0) {
?>
<br />
<h3><?php printf(SERENDIPITY_UPGRADER_DATABASE_UPDATES, $serendipity['dbType']) ?>:</h3>
<?php echo SERENDIPITY_UPGRADER_FOUND_SQL_FILES ?>:
<br />
<?php
        foreach ($sqlfiles as $sqlfile) {
            echo '<strong>- '. $sqlfile .'</strong><br />';
        }
?>
<?php
    }
?>
<br />


<h3><?php echo SERENDIPITY_UPGRADER_VERSION_SPECIFIC ?>:</h3>
<?php
    $taskCount = 0;

    if (version_compare(serendipity_version($serendipity['versionInstalled']), '0.5.1', '<')) {
        echo '<br /><strong>0.5.1 - Image sync</strong>';
        echo '<br />Version 0.5.1 introduces image sync with the database.';
        echo '<br />With your permission I would like to perform the image sync.<br />';
        $taskCount++;
    }

    if (version_compare(serendipity_version($serendipity['versionInstalled']), '0.6', '<')) {
        echo '<br /><strong>0.6 - Move images</strong>';
        echo '<br />[INFO] Smilies and XML-Buttons were moved from pixel/ to template/default/img<br />';

        echo '<br /><strong>0.6 - Template/CSS changes</strong>';
        echo '<br />[INFO] A home-link has been added to the page headers. For that, two new CSS classes have been introduced: #homelink1 (header), homelink2 (subheader).<br />';
        echo 'If you have a custom template, you need to adapt to those changes. To make themlook like your header previously has, insert this in your style.css:<br /><br />';

        echo '<pre>a.homelink1,
  a.homelink1:hover,
  a.homelink1:link,
  a.homelink1:visited,
  #serendipity_banner h1 {
    ... your css-definitions ...
    text-decoration: none;
  }

  a.homelink2,
  a.homelink2:hover,
  a.homelink2:link,
  a.homelink2:visited,
  #serendipity_banner h2 {
    ... your css-definitions ...
    text-decoration: none;
  }</pre><br />';
    }

    if (version_compare(serendipity_version($serendipity['versionInstalled']), '0.6.5', '<')) {
        echo '<br /><strong>0.6.5 - Nested subcategories, post to multiple categories</strong>';
        echo '<br />[<strong style="color: red">INFO</strong>] This update will update the categories table of your database and update the relations from entries to categories. This is a possibly dangerous task to perform, so <strong style="color: red">make sure you have a backup of your database!</strong><br />';
        $taskCount++;
    }

    if (version_compare(serendipity_version($serendipity['versionInstalled']), '0.6.8', '<')) {
        echo '<br /><strong>0.6.8 - .htaccess</strong>';
        echo '<br />[INFO] The Plugin API now allows external plugins to generate individual content. Your .htaccess file needs to be updated for this to occur. This will happen automatically, if you continue. If you made individual changes to your .htaccess file, backup that file now.<br />';
        $taskCount++;
    }

    if (version_compare(serendipity_version($serendipity['versionInstalled']), '0.6.10', '<')) {
        echo '<br /><strong>0.6.10 - .htaccess</strong>';
        echo '<br />[INFO] Your .htaccess file needs to be updated for this to occur. This will happen automatically, if you continue. If you made individual changes to your .htaccess file, backup that file now.<br />';
        $taskCount++;
    }

    if (version_compare(serendipity_version($serendipity['versionInstalled']), '0.6.12', '<')) {
        echo '<br /><strong>0.6.12 - .htaccess</strong>';
        echo '<br />[INFO] Your .htaccess file needs to be updated for this to occur. This will happen automatically, if you continue. If you made individual changes to your .htaccess file, backup that file now.<br />';
        $taskCount++;
    }



    /*

    Add more versions here using:

        if ([version_compare]) {
            [Desc]
            $taskCount++;
        }

    */

    if ($taskCount == 0) {
        echo SERENDIPITY_UPGRADER_NO_VERSION_SPECIFIC;
    }
?>

<br /><br />
<hr noshade="noshade">
<?php if ($taskCount > 0 || sizeof($sqlfiles) > 0) { ?>
    <strong><?php echo SERENDIPITY_UPGRADER_PROCEED_QUESTION ?></strong>
    <br /><?php if ($showAbort) { ?><input type="button" value="<?php echo SERENDIPITY_UPGRADER_PROCEED_ABORT ?>" onclick="location.href='<?php echo $abortLoc ?>'"> <?php } ?><input type="button" value="<?php echo SERENDIPITY_UPGRADER_PROCEED_DOIT ?>" onclick="location.href='<?php echo $upgradeLoc ?>'">
<?php } else { ?>
    <strong><?php echo SERENDIPITY_UPGRADER_NO_UPGRADES ?></strong>
    <br /><input type="button" value="<?php echo SERENDIPITY_UPGRADER_CONSIDER_DONE ?>" onclick="location.href='<?php echo $abortLoc ?>'">
<?php }
}
?>
