<?php # $Id: serendipity_db_postgres.inc.php,v 1.18 2004/09/09 10:52:02 garvinhicking Exp $

function serendipity_db_connect() {
    global $serendipity;

    $serendipity['dbConn'] = pg_connect(
                               sprintf(
                                 '%sdbname=%s user=%s password=%s',

                                 strlen($serendipity['dbHost']) ? ('host=' . $serendipity['dbHost'] . ' ') : '',
                                 $serendipity['dbName'],
                                 $serendipity['dbUser'],
                                 $serendipity['dbPass']
                               )
                             );

    return $serendipity['dbConn'];
}

function serendipity_db_escape_string($string) {
    return pg_escape_string($string);
}

function serendipity_db_limit($start, $offset) {
    return $offset . ', ' . $start;
}

function serendipity_db_limit_sql($limitstring) {
    $limit_split = split(',', $limitstring);
    if (count($limit_split) > 1) {
        $limit = ' LIMIT ' . $limit_split[0] . ' OFFSET ' . $limit_split[1];
    } else {
        $limit = ' LIMIT ' . $limit_split[0];
    }
    return $limit;
}

function serendipity_db_affected_rows() {
    global $serendipity;
    return pg_affected_rows($serendipity['dbLastResult']);
}

function serendipity_db_insert_id($table = '', $id = '') {
    global $serendipity;
    if (empty($table) || empty($id)) {
        return pg_last_oid($serendipity['dbLastResult']);
    } else {
        $oid   = pg_last_oid($serendipity['dbLastResult']);
        $query = "SELECT $id FROM {$serendipity['dbPrefix']}$table WHERE oid=$oid";
        $res   = pg_query($serendipity['dbConn'], $query);
        if (pg_num_rows($res)) {
            $insert_id = pg_fetch_array($res, 0, PGSQL_ASSOC);
            return $insert_id[$id];
        } else {
            return $oid;
        }
    }
}

function serendipity_db_query($sql, $single = false, $result_type = "both", $reportErr=false, $assocKey=false, $assocVal=false) {
    global $serendipity;
    static $type_map = array(
                         'assoc' => PGSQL_ASSOC,
                         'num'   => PGSQL_NUM,
                         'both'  => PGSQL_BOTH
    );

    $serendipity['dbLastResult'] = pg_query($serendipity['dbConn'], $sql);

    if (!$serendipity['dbLastResult']) {
        if (!$serendipity['production']) {
            print "Error in $sql<br/>\n";
            print pg_last_error($serendipity['dbConn']) . "<BR/>\n";
            if (function_exists('debug_backtrace')) {
                highlight_string(var_export(debug_backtrace(), 1));
            }
            print "<br><code>$sql</code>\n";
        }
        return false;
    }

    if ($serendipity['dbLastResult'] === true) {
        return true;
    }

    $result_type = $type_map[$result_type];

    $n = pg_num_rows($serendipity['dbLastResult']);

    switch ($n) {
        case 0:
            if ($single) {
                return false;
            }
            return true;
        case 1:
            if ($single) {
                return pg_fetch_array($serendipity['dbLastResult'], 0, $result_type);
            }
        default:
            $rows = array();
            for ($i = 0; $i < $n; $i++) {
                if (!empty($assocKey) && !empty($assocVal)) {
                    // You can fetch a key-associated array via the two function parameters assocKey and assocVal
                    $row = pg_fetch_array($serendipity['dbLastResult'], $i, $result_type);
                    $rows[$row[$assocKey]] = $row[$assocVal];
                } else {
                    $rows[] = pg_fetch_array($serendipity['dbLastResult'], $i, $result_type);
                }
            }
            return $rows;
    }
}

function serendipity_db_schema_import($query) {
    static $search  = array('{AUTOINCREMENT}', '{PRIMARY}', '{UNSIGNED}',
        '{FULLTEXT}', '{BOOLEAN}', 'int(1)', 'int(10)', 'int(11)', 'int(4)');
    static $replace = array('SERIAL', 'primary key', '', '', 'BOOLEAN NOT NULL', 'int2',
        'int4', 'int4', 'int4');

    return serendipity_db_query(str_replace($search, $replace, $query));
}

function serendipity_db_probe($hash, &$errs) {
    global $serendipity;

    if (!function_exists('pg_connect')) {
        $errs[] = 'No PostgreSQL extension found. Please check your webserver installation or contact your systems administrator regarding this problem.';
        return false;
    }

    $serendipity['dbConn'] = pg_connect(
                               sprintf(
                                 '%sdbname=%s user=%s password=%s',

                                 strlen($hash['dbHost']) ? ('host=' . $hash['dbHost'] . ' ') : '',
                                 $hash['dbName'],
                                 $hash['dbUser'],
                                 $hash['dbPass']
                               )
                             );

    if (!$serendipity['dbConn']) {
        $errs[] = 'Could not connect to database; check your settings.';
        return false;
    }

    return true;
}

function serendipity_db_concat($string) {
    return 'concat_text(' . $string . ')';
}

/* vim: set sts=4 ts=4 expandtab : */
?>
