<?php # $Id: serendipity_db_sqlite.inc.php,v 1.10 2004/08/30 09:29:23 garvinhicking Exp $

if (!function_exists('sqlite_open')) {
    @dl('sqlite.so');
    @dl('sqlite.dll');
}

function serendipity_db_connect()
{
    global $serendipity;

    if (isset($serendipity['dbConn'])) {
        return $serendipity['dbConn'];
    }

    $serendipity['dbConn'] = sqlite_open($serendipity['serendipityPath'] . $serendipity['dbName'] . '.db');

    return $serendipity['dbConn'];
}

function serendipity_db_escape_string($string)
{
    static $search  = array("\x00", '%',   "'",   '\"');
    static $replace = array('%00',  '%25', "''", '\\\"');

    return str_replace($search, $replace, $string);
}

function serendipity_db_affected_rows()
{
    global $serendipity;

    return sqlite_changes($serendipity['dbConn']);
}

function serendipity_db_insert_id()
{
    global $serendipity;

    return sqlite_last_insert_rowid($serendipity['dbConn']);
}

function serendipity_db_sqlite_fetch_array($res, $type = SQLITE_BOTH)
{
    static $search  = array('%00',  '%25');
    static $replace = array("\x00", '%');

    $row = sqlite_fetch_array($res, $type);
    if (!is_array($row)) {
        return $row;
    }

    /* strip any slashes, correct fieldname */
    foreach ($row as $i => $v) {
        // TODO: If a query of the format 'SELECT a.id, b.text FROM table' is used,
        //       the sqlite extension will give us key indizes 'a.id' and 'b.text'
        //       instead of just 'id' and 'text' like in mysql/postgresql extension.
        //       To fix that, we use a preg-regex; but that is quite performance costy.
        //       Either we always need to use 'SELECT a.id AS id, b.text AS text' in query,
        //       or the sqlite extension may get fixed. :-)
        $row[preg_replace('@^.+\.(.*)@', '\1', $i)] = str_replace($search, $replace, $v);
    }

    return $row;
}

function serendipity_db_query($sql, $single = false, $result_type = "both", $reportErr=true, $assocKey=false, $assocVal=false)
{
    global $serendipity;
    static $type_map = array(
                         'assoc' => SQLITE_ASSOC,
                         'num'   => SQLITE_NUM,
                         'both'  => SQLITE_BOTH
    );

    static $debug = false;

    if ($debug) $fp = @fopen('sqlite.log', 'a');

    if ($reportErr) {
        $res = sqlite_query($sql, $serendipity['dbConn']);
    } else {
        $res = @sqlite_query($sql, $serendipity['dbConn']);
    }

    if (!$res) {
        if (!$serendipity['production']) {
            var_dump($res);
            var_dump($sql);
            return "problem with query";
        }
        if ($debug) fwrite($fp, '[' . date('d.m.Y H:i') . '] [ERROR] SQLITE QUERY: ' . $sql . "\n\n");

        return false;
    } elseif ($debug) {
        fwrite($fp, '[' . date('d.m.Y H:i') . '] SQLITE QUERY: ' . $sql . "\n\n");
    }

    if ($res === true) {
        return true;
    }

    if (sqlite_num_rows($res) == 0) {
        if ($single) {
            return false;
        }
        return true;
    } else {
        $rows = array();

        while (($row = serendipity_db_sqlite_fetch_array($res, $type_map[$result_type]))) {
            if (!empty($assocKey) && !empty($assocVal)) {
                // You can fetch a key-associated array via the two function parameters assocKey and assocVal
                $rows[$row[$assocKey]] = $row[$assocVal];
            } else {
                $rows[] = $row;
            }
        }

        if ($debug) fwrite($fp, '[' . date('d.m.Y H:i') . '] SQLITE RESULT: ' . print_r($rows, true). "\n\n");
        if ($debug) fclose($fp);

        if ($single && count($rows) == 1) {
            return $rows[0];
        }

        return $rows;
    }
}

function serendipity_db_probe($hash, &$errs)
{
    global $serendipity;

    $dbName = (isset($hash['sqlitedbName']) ? $hash['sqlitedbName'] : $hash['dbName']);

    if (!function_exists('sqlite_open')) {
        $errs[] = 'SQLite extension not installed. Run "pear install sqlite" on your webserver or contact your systems administrator regarding this problem.';
        return false;
    }

    $serendipity['dbConn'] = sqlite_open($serendipity['serendipityPath'] . $dbName . '.db');

    if ($serendipity['dbConn']) {
        return true;
    }

    $errs[] = "Unable to open \"{$serendipity['serendipityPath']}$dbName.db\" - check permissions (directory needs to be writeable for webserver)!";
    return false;
}

function serendipity_db_schema_import($query)
{
    static $search  = array('{AUTOINCREMENT}', '{PRIMARY}', '{UNSIGNED}', '{FULLTEXT}', '{BOOLEAN}');
    static $replace = array('INTEGER', 'PRIMARY KEY', '', '', 'BOOLEAN NOT NULL');

    return serendipity_db_query(str_replace($search, $replace, $query));
}

function serendipity_db_limit($start, $offset) {
    return $start . ', ' . $offset;
}

function serendipity_db_limit_sql($limitstring) {
    return ' LIMIT ' . $limitstring;
}

function serendipity_db_concat($string) {
    return 'concat(' . $string . ')';
}

/* vim: set sts=4 ts=4 expandtab : */
?>
