<?php # $Id: serendipity_plugin_api.php,v 1.23 2004/06/17 20:57:10 garvinhicking Exp $

if (IN_serendipity !== true) {
    die ('Don\'t hack!');
}

include_once S9Y_INCLUDE_PATH . 'serendipity_functions.inc.php';

/* This file defines the plugin API for serendipity.
 * By extending these classes, you can add your own code
 * to appear in the sidebar(s) of serendipity.
 *
 *
 * The system defines a number of built-in plugins; these are
 * identified by @class_name.
 *
 * Third-party plugins are identified by the name of the folder into
 * which they were uploaded (so there is no @ sign at the start of
 * their class name.
 *
 * The user creates instances of plugins; an instance is assigned
 * an identifier like this:
 *   classname:uniqid()
 *
 * The user can configure instances of plugins.
 */

class serendipity_plugin_api {

    function register_default_plugins()
    {
        $classes = serendipity_plugin_api::enum_plugin_classes();
        foreach ($classes as $class_name) {
            if ($class_name{0} == '@') {
                serendipity_plugin_api::create_plugin_instance($class_name);
            }
        }

        serendipity_plugin_api::create_plugin_instance('serendipity_event_s9ymarkup', null, 'event');
        serendipity_plugin_api::create_plugin_instance('serendipity_event_emoticate', null, 'event');
        serendipity_plugin_api::create_plugin_instance('serendipity_event_nl2br', null, 'event');
        serendipity_plugin_api::create_plugin_instance('serendipity_event_trackexits', null, 'event');
    }

    /* Create an instance of a plugin.
     * $plugin_class_id is of the form:
     *    @class_name        for a built-in plugin
     * or
     *    plugin_dir_name    for a third-party plugin
     * returns the instance identifier for the newly created plugin.
     *
     * TO BE IMPLEMENTED:
     * If $copy_from_instance is not null, and identifies another plugin
     * of the same class, then the persistent state will be copied.
     * This allows the user to clone a plugin.
     */
    function create_plugin_instance($plugin_class_id, $copy_from_instance = null, $default_placement = 'right', $authorid = '0')
    {
        global $serendipity;

        $id = md5(uniqid(''));

        $key = $plugin_class_id . ':' . $id;

        $rs = serendipity_db_query("SELECT MAX(sort_order) as sort_order_max FROM {$serendipity['dbPrefix']}plugins WHERE placement = '$default_placement'", true, 'num');

        if (is_array($rs)) {
            $nextidx = intval($rs[0]+1);
        } else {
            $nextidx = 0;
        }

        serendipity_db_query("INSERT INTO {$serendipity['dbPrefix']}plugins (name, sort_order, placement, authorid) values ('$key', $nextidx, '$default_placement', '$authorid')");

        /* Check for multiple dependencies */
        $plugin =& serendipity_plugin_api::load_plugin($key);
        $bag    = new serendipity_property_bag;
        $plugin->introspect($bag);
        $plugin->register_dependencies(false, $authorid);

        return $key;
    }

    function remove_plugin_instance($plugin_instance_id)
    {
        global $serendipity;

        $plugin =& serendipity_plugin_api::load_plugin($plugin_instance_id);
        $bag    = new serendipity_property_bag;
        $plugin->introspect($bag);

        serendipity_db_query("DELETE FROM {$serendipity['dbPrefix']}plugins where name='$plugin_instance_id'");

        $plugin->register_dependencies(true);

        serendipity_db_query("DELETE FROM {$serendipity['dbPrefix']}config  where name LIKE '$plugin_instance_id/%'");
    }

    function remove_plugin_value($plugin_instance_id, $where)
    {
        global $serendipity;
        $where_sql = array();
        foreach($where AS $key) {
            $where_sql[] = "(name LIKE '{$plugin_instance_id}/{$key}_%' AND value = '')";
        }

        $query = "DELETE FROM  {$serendipity['dbPrefix']}config
                                    WHERE  " . implode(' OR ', $where_sql);

        serendipity_db_query($query);
    }

    /* Retrieves a list of available plugins */
    function enum_plugin_classes($event_only = false)
    {
        global $serendipity;

        $classes = array();

        /* built-in classes first */
        $cls = get_declared_classes();
        foreach ($cls as $class_name) {
            if (strncmp($class_name, 'serendipity_', 6)) {
                continue;
            }

            $p = get_parent_class($class_name);
            while ($p != 'serendipity_plugin' && $p != 'serendipity_event' && $p !== false) {
                $p = get_parent_class($p);
            }

            if ($p == 'serendipity_plugin' && $class_name != 'serendipity_event' && !$event_only ) {
                $classes[] = '@' . $class_name;
            } elseif ($p == 'serendipity_event' && $class_name != 'serendipity_event' && $event_only) {
                $classes[] = '@' . $class_name;
            }
        }

        /* GLOBAL third-party classes next */
        $ppath = dirname(__FILE__) . '/plugins';
        $d = opendir($ppath);
        if ($d) {
            while (($f = readdir($d)) !== false) {
                if ($f{0} == '.' || $f == 'CVS') {
                    continue;
                }

                if ($event_only && !serendipity_plugin_api::is_event_plugin($f)) {
                    continue;
                }

                if (!$event_only && serendipity_plugin_api::is_event_plugin($f)) {
                    continue;
                }

                $existing = array_search('@' . $f, $classes);
                if ($existing !== null && $existing !== false) {
                    // If an external plugin/event already exists as internal, remove the internal reference because its redundant
                    unset($classes[$existing]);
                }

                if (file_exists($ppath . '/' . $f . '/' . $f . '.php')) {
                    $classes[] = $f;
                }
            }
        }

        /* LOCAL third-party classes next */
        $local_ppath = $serendipity['serendipityPath'] . 'plugins';
        if ($ppath != $local_ppath) {
            $d = @opendir($local_ppath);
            if ($d) {
                while (($f = readdir($d)) !== false) {
                    if ($f{0} == '.' || $f == 'CVS') {
                        continue;
                    }

                    if ($event_only && !serendipity_plugin_api::is_event_plugin($f)) {
                        continue;
                    }

                    if (!$event_only && serendipity_plugin_api::is_event_plugin($f)) {
                        continue;
                    }

                    $existing = array_search('@' . $f, $classes);
                    if ($existing !== null && $existing !== false) {
                        // If an external plugin/event already exists as internal, remove the internal reference because its redundant
                        unset($classes[$existing]);
                    }

                    $existing = array_search($f, $classes);
                    if ($existing !== null && $existing !== false) {
                        // A local plugin will be preferred over general plugins
                        unset($classes[$existing]);
                    }

                    if (file_exists($local_ppath . '/' . $f . '/' . $f . '.php')) {
                        $classes[] = $f;
                    }
                }
            }
        }

        return $classes;
    }

    /* Retrieves a list of plugin instances */
    function enum_plugins($filter = '*', $negate = false)
    {
        global $serendipity;

        $sql = "SELECT * from {$serendipity['dbPrefix']}plugins ";

        if ($filter !== '*') {
            if ($negate) {
                $sql .= "WHERE placement != '$filter' ";
            } else {
                $sql .= "WHERE placement='$filter' ";
            }
        }

        $sql .= ' ORDER BY placement, sort_order';

        return serendipity_db_query($sql);
    }

    /* Creates an instance of a named plugin */
    function &load_plugin($instance_id)
    {
        global $serendipity;

        $instance = explode(':', $instance_id);
        $name     = $instance[0];

        if ($name{0} == '@') {
            $class_name = substr($name, 1);
        } else {
            /* plugin from the plugins/ dir */
            if (!class_exists($name)) {
                // First try the local path, and then (if existing) a shared library repository ...
                if (file_exists($serendipity['serendipityPath'] . 'plugins/' . $name . '/' . $name . '.php')) {
                    include $serendipity['serendipityPath'] . 'plugins/' . $name . '/' . $name . '.php';
                } elseif (file_exists(S9Y_INCLUDE_PATH . 'plugins/' . $name . '/' . $name . '.php')) {
                    include S9Y_INCLUDE_PATH . 'plugins/' . $name . '/' . $name . '.php';
                }

                if (!class_exists($name)) {
                    return false;
                }
            }

            $class_name = $name;
        }

        $p =& new $class_name($instance_id);

        $sql = "SELECT authorid from {$serendipity['dbPrefix']}plugins WHERE name = '" . $instance_id . "'";
        $owner = serendipity_db_query($sql, true);
        $p->serendipity_owner = $owner[0];

        return $p;
    }

    function update_plugin_placement($name, $placement, $order=null)
    {
        global $serendipity;

        $sql = "UPDATE {$serendipity['dbPrefix']}plugins set placement='$placement' ";

        if ($order !== null) {
            $sql .= ", sort_order=$order ";
        }

        $sql .= "WHERE name='$name'";

        return serendipity_db_query($sql);
    }

    function update_plugin_owner($name, $authorid)
    {
        global $serendipity;

        if (empty($authorid) && $authorid != '0') {
            return;
        }

        $admin = '';
        if ($serendipity['serendipityUserlevel'] < USERLEVEL_ADMIN) {
            $admin = " AND (authorid = 0 OR authorid = {$serendipity['authorid']})";
        }

        $sql = "UPDATE {$serendipity['dbPrefix']}plugins SET authorid='$authorid' WHERE name='$name' $admin";

        return serendipity_db_query($sql);
    }

    function generate_plugins($side, $tagname = 'div')
    {
        $plugins = serendipity_plugin_api::enum_plugins($side);

        if (!is_array($plugins)) {
            return;
        }

        echo "<$tagname id=\"serendipity" . ucfirst($side) . "SideBar\">\n";
        foreach ($plugins as $plugin_data) {
            $plugin =& serendipity_plugin_api::load_plugin($plugin_data['name']);
            $title  = '';

            ob_start();
            if ($plugin->generate_content($title) !== false) {
                $content = ob_get_contents();
                $class   = get_class($plugin);
                ob_end_clean();

                echo "    <div class=\"$plugin->wrap_class sidebar_wrap_$class\">\n";
                if (!empty($title)) {
                    echo "        <h3 class=\"$plugin->title_class sidebar_title_$class\">$title</h3>\n";
                }
                echo "        <div class=\"$plugin->content_class sidebar_content_$class\">\n";
                echo "            $content\n";
                echo "        </div>\n";
                echo "    </div>\n";
            } else {
                ob_end_clean();
            }
        }

        echo "</$tagname>\n";
    }

    function get_plugin_title(&$plugin, $default_title = '')
    {
        global $serendipity;

        // Generate plugin output. Make sure that by probing the plugin, no events are actually called. After that,
        // restore setting of 'no_events'.

        $ne = (isset($serendipity['no_events']) && $serendipity['no_events'] ? TRUE : FALSE);
        $serendipity['no_events'] = TRUE;
        ob_start();
        $plugin->generate_content($title);
        ob_end_clean();
        $serendipity['no_events'] = $ne;

        if (strlen(trim($title)) == 0) {
            if (!empty($default_title)) {
                $title = $default_title;
            } else {
                $title = $plugin->instance;
            }
        }

        return $title;
    }

    /* conditional to see if the named plugin is an event plugin
        Refactoring: decompose conditional */
    function is_event_plugin($name) {
        return (strstr($name, '_event_'));
    }

    function &get_event_plugins($instance = false) {
        global $serendipity;

        if (!isset($serendipity['event_plugins']) || !is_array($serendipity['event_plugins'])) {
            $plugins = serendipity_plugin_api::enum_plugins('event');
            $serendipity['event_plugins'] = array();

            if (!is_array($plugins)) {
                return false;
            }

            foreach ($plugins as $plugin_data) {
                if ($serendipity['event_plugins'][$plugin_data['name']]['p'] =& serendipity_plugin_api::load_plugin($plugin_data['name'])) {
                    /* query for its name, description and configuration data */
                    $serendipity['event_plugins'][$plugin_data['name']]['b'] = new serendipity_property_bag;
                    $serendipity['event_plugins'][$plugin_data['name']]['p']->introspect($serendipity['event_plugins'][$plugin_data['name']]['b']);
                    $serendipity['event_plugins'][$plugin_data['name']]['t'] = serendipity_plugin_api::get_plugin_title($serendipity['event_plugins'][$plugin_data['name']]['p']);
                }
            }
        }

        if ($instance) {
            if (isset($serendipity['event_plugins'][$instance]['p'])) {
                return $serendipity['event_plugins'][$instance]['p'];
            } else {
                return false;
            }
        } else {
            return $serendipity['event_plugins'];
        }
    }

    function hook_event($event_name, &$eventData, $addData = null) {
        global $serendipity;

        // Can be bypassed globally
        // $serendipity['no_events'] = TRUE;
        if (!isset($serendipity['no_events']) || !$serendipity['no_events']) {
            $plugins = &serendipity_plugin_api::get_event_plugins();

            if (is_array($plugins)) {
                foreach($plugins AS $plugin => $plugin_data) {
                    $bag = &$plugin_data['b'];
                    $plugin_data['p']->event_hook($event_name, $plugin_data['b'], $eventData, $addData);
                }
            }
        }

        return true;
    }

    function exists($instance_id) {
        global $serendipity;

        if (!strstr($instance_id, ':')) {
            $instance_id .= ':';
        }

        $existing = serendipity_db_query("SELECT name FROM {$serendipity['dbPrefix']}plugins WHERE name LIKE '%$instance_id%'");

        if (is_array($existing) && !empty($existing[0][0])) {
            return $existing[0][0];
        }

        return false;
    }
}

/* holds a bunch of properties; you can have multiple
* properties with the same name */
class serendipity_property_bag {
    var $properties = array();
    var $name       = null;

    function add($name, $value)
    {
        $this->properties[] = array(
                                'name'  => $name,
                                'value' => $value
        );
    }

    function get($name)
    {
        $v = array();
        foreach ($this->properties as $data) {
            if ($data['name'] == $name) {
                $v[] = $data['value'];
            }
        }

        if (count($v) == 1) {
            return $v[0];
        }

        return implode(', ', $v);
    }

    function is_set($name)
    {
        foreach ($this->properties as $data) {
            if ($data['name'] == $name) {
                return true;
            }
        }

        return false;
    }
}

class serendipity_plugin {
    var $instance      = null;
    var $protected     = false;
    var $wrap_class    = 'serendipitySideBarItem';
    var $title_class   = 'serendipitySideBarTitle';
    var $content_class = 'serendipitySideBarContent';

    /* Be sure to call this method from your derived classes constructors,
     * otherwise your config data will not be stored or retrieved correctly
     */

    function serendipity_plugin($instance)
    {
        $this->instance = $instance;
    }

    /* Called by serendipity when it wants to display information
     * about your plugin.
     * You need to override this method in your child class.
     */
    function introspect(&$propbag)
    {
        $propbag->add('copyright', 'MIT License');
        $propbag->add('name'     , get_class($this));

        // $propbag->add(
        //   'configuration',
        //   array(
        //     'text field',
        //     'foo bar'
        //   )
        // );

        $this->protected = FALSE; // If set to TRUE, only allows the owner of the plugin to modify its configuration

        return true;
    }

    /* Called by serendipity when it wants to display the configuration
     * editor for your plugin.
     * $name is the name of a configuration item you added in
     * your instrospect method.
     * You need to fill the property bag with appropriate items
     * that describe the type and value(s) for that particular
     * configuration option.
     * You need to override this method in your child class if
     * you have configuration options.
     */
    function introspect_config_item($name, &$propbag)
    {
        return false;
    }

    /* Called by serendipity when it wants your plugin to display itself.
     * You need to set $title to be whatever text you want want to
     * appear in the item caption space.
     * Simply echo/print your content to the output; serendipity will
     * capture it and make things work.
     * You need to override this method in your child class.
     */
    function generate_content(&$title)
    {
        $title = 'Sample!';
        echo     'This is a sample!';
    }

    /* Fetches a configuration value for this plugin */
    function get_config($name, $defaultvalue = null, $empty = true)
    {
        $name = $this->instance . '/' . $name;
        return serendipity_get_config_var($name, $defaultvalue, $empty);
    }

    function set_config($name, $value)
    {
        $name = $this->instance . '/' . $name;

        return serendipity_set_config_var($name, $value);
    }

    /* Called by serendipity after insertion of a config item. If you want to kick out certain
     * elements based on contents, create the corresponding function here.
     */
    function cleanup()
    {
        // Cleanup. Remove all empty configs on SAVECONF-Submit.
        // serendipity_plugin_api::remove_plugin_value($this->instance, array('title', 'description'));
        return true;
    }

    function register_dependencies($remove = false, $authorid = '0')
    {
        global $serendipity;

        if (isset($this->dependencies) && is_array($this->dependencies)) {

            if ($remove) {
                $dependencies = @explode(';', $this->get_config('dependencies'));
                $modes        = @explode(';', $this->get_config('dependency_modes'));

                if (!empty($dependencies) && is_array($dependencies)) {
                    foreach($dependencies AS $idx => $dependency) {
                        if ($modes[$idx] == 'remove' && serendipity_plugin_api::exists($dependency)) {
                            serendipity_plugin_api::remove_plugin_instance($dependency);
                        }
                    }
                }
            } else {
                $keys  = array();
                $modes = array();
                foreach($this->dependencies AS $dependency => $mode) {
                    $exists = serendipity_plugin_api::exists($dependency);
                    if (!$exists) {
                        if (serendipity_plugin_api::is_event_plugin($dependency)) {
                            $keys[] = serendipity_plugin_api::create_plugin_instance($dependency, null, 'event', $authorid);
                        } else {
                            $keys[] = serendipity_plugin_api::create_plugin_instance($dependency, null, 'right', $authorid);
                        }
                    } else {
                        $keys[] = $exists;
                    }

                    $modes[] = $mode;
                }

                $this->set_config('dependencies',     implode(';', $keys));
                $this->set_config('dependency_modes', implode(';', $modes));
            }
        }

        return true;
    }
}

class serendipity_event extends serendipity_plugin {
    /* Events can be called on several occasions when s9y performs an action.
     * One or multiple plugin can be registered for each of those hooks.
     */

    /* Be sure to call this method from your derived classes constructors,
     * otherwise your config data will not be stored or retrieved correctly
     */

    function serendipity_event($instance)
    {
        $this->instance = $instance;
    }

    function event_hook($event, &$bag, &$eventData, $addData = null) {
        // Define event hooks here, if you want you plugin to execute those instead of being a sidebar item.
        // Look at external plugins 'serendipity_event_mailer' or 'serendipity_event_weblogping' for usage.
        // Currently available events:
        //   backend_publish [after insertion of a new article in your s9y-backend]
        //   backend_display [after displaying an article in your s9y-backend]
        //   frontend_display [before displaying an article in your s9y-frontend]
        //   frontend_comment [after displaying the "enter comment" dialog]
        return true;
    }
}

include_once S9Y_INCLUDE_PATH . 'serendipity_sidebar_items.php';

/* vim: set sts=4 ts=4 expandtab : */
?>
